Package: SFOCDs
Type: Package
Title: Space Filling Optimal Covariate Designs
Version: 1.2.0
Authors@R: c(person("Neethu", "RS", email = "neethurs37@gmail.com", role =c( "aut","ctb")),person("Cini","Varghese",email="cini2204@gmail.com",role=c("aut","ctb")),person("Mohd", "Harun", email = "harun.agribhu@gmail.com" , role = c( "aut","ctb")),person("Anindita", "Datta", email = "aninditaiasri@gmail.com", role =c( "aut","ctb")),person("Ashutosh","Dalal", email = "ashutosh.dalal97@gmail.com", role =c( "aut","ctb", "cre")))
Description: We have designed this package to address experimental scenarios involving multiple covariates. It focuses on construction of Optimal Covariate Designs (OCDs), checking space filling property of the developed design. The primary objective of the package is to generate OCDs using four methods viz., M array method, Juxtapose method, Orthogonal Integer Array and Hadamard method. The package also evaluates space filling properties of both the base design and OCDs using the MaxPro criterion, providing a meaningful basis for comparison. In addition, it includes tool to visualize the spread offered by the design points in the form of scatterplot, which help users to assess distribution and coverage of design points.
Maintainer: Ashutosh Dalal <ashutosh.dalal97@gmail.com>
Suggests: gtools
License: GPL (>= 2)
Encoding: UTF-8
Repository: CRAN
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-04-28 17:40:20 UTC; Ashutosh
Author: Neethu RS [aut, ctb],
  Cini Varghese [aut, ctb],
  Mohd Harun [aut, ctb],
  Anindita Datta [aut, ctb],
  Ashutosh Dalal [aut, ctb, cre]
Date/Publication: 2026-04-29 11:50:06 UTC
Built: R 4.6.0; ; 2026-04-29 15:57:53 UTC; unix

# Generated by roxygen2: do not edit by hand

export(HadamardOCDs)
export(JuxtaOCDs)
export(MOCDs)
export(MaxDot)
export(Maxpro_Criterion)
export(OIAOCDs)
export(reshuffle_des)
importFrom(graphics,axis)
importFrom(utils,combn)

#' Juxtaposed Method for Optimal Covariate Designs (OCDs)
#'
#' @param design Input a design in matrix format and block size k such that (k+1) is a prime number.
#'@description
#'Consider L matrix, construct resolvable sets by grouping columns into pairs that have the same ordered set of elements. For each pair, arrange the two column-sets horizontally (2(s-1)) and change the signs of any two sets. This new setup form the P_i matrix of order 2(s-1) x 2. Then superimpose the first column of P_i onto N and that produce W_i, where first set of order v x b will be W_i^11 and other set below is W_i^21. Likewise use second column of P_i  to  get W_i^12 and W_i^22.  Repeat for every P_i to get collection of W_i's. The grand total of Hadamard product of all W_i^ij will be zero provided a foldover of any one of the W_i^ij is taken.
#'
#' @returns Generates W matrices and Inter product sums of W matrices.
#' @references
#' Das, K., N. K. Mandal, and B. K. Sinha. (2003) <https://doi.org/10.1016/S0378-3758(02)00151-9>. Optimal experimental designs for models with covariates. Journal of Statistical Planning and Inference 115(1): 273-285.
#'
#' Bansal, N., and D. K. Garg. (2022)<https://doi.org/10.1007/s42519-022-00244-0>. Optimum covariate designs for three associate PBIB designs. Journal of Statistical Theory and Practice 16(3): 1-15.
#'
#' @export
#'@author
#'Neethu RS
#'
#'Cini Varghese
#'
#'Mohd Harun
#'
#'Anindita Datta
#'
#'Ashutosh Dalal
#'
#' @examples
#' library(SFOCDs)
#' mat1<-matrix(c(
#' 1,2,3,4,5,6,
#' 7,8,9,10,11,1,
#' 12,13,14,15,2,7,
#' 16,17,18,3,8,12,
#' 19,20,4,9,13,16,
#' 21,5,10,14,17,19,
#' 6,11,15,18,20,21),nrow=7,byrow=TRUE)
#' JuxtaOCDs(mat1)
JuxtaOCDs<-function(design){
  design<-as.matrix(design)
  block <- nrow(design)
  trt <- max(design)
  mat=design
  incidence_mat<-function(mat){
    mat<-as.matrix(mat)
    incidence_matrix <- matrix(0, nrow = nrow(mat), ncol = max(c(mat)))
    for (i in 1:nrow(mat)) {
      #for (j in 1:max(c(mat))) {
      incidence_matrix[i, c(mat[i, ])] <- 1
      #}
    }
    return(incidence_matrix)
  }
  incidence_matrix_transpose <- t(incidence_mat(design))
  inc<-incidence_matrix_transpose
  #write.table(inc,"clipboard",sep="\t")

  MOLS<-function(v){
    MOLS<-list()
    for(i in 1:(v-1)){
      mols<-NULL
      seq<-c(1:v)
      j=0
      repeat{
        mols<-rbind(mols,c(seq+j))
        j=j+i
        if(nrow(mols)==v){
          mols=mols%%v
          mols[mols==0]<-v
          MOLS<-append(MOLS,list(mols))
          break
        }
      }
    }
    return(MOLS)
  }
  k=ncol(design)
  mols<-MOLS(k+1)
  del_col_mols<-lapply(mols,function(mat)mat[,1])
  L<-do.call(cbind,del_col_mols)
  L
  L<-L[-1,]
  L<-L-1
  #####pmatrix
  same_pairs <- function(A, B) {
    sort_rows <- function(mat) {
      t(apply(mat, 1, sort))
    }

    A<-sort_rows(A)
    B<-sort_rows(B)
    # Convert rows to character strings (unique row identifiers)
    A_rows <- apply(A, 1, paste, collapse = "_")
    B_rows <- apply(B, 1, paste, collapse = "_")

    # Compare sorted row sets
    identical(sort(A_rows), sort(B_rows))
  }
  ###difference vector
  diff<-function(mat){
    A<-mat[,2]-mat[,1]
    return(A)
  }
  ##first pair of column fix for P1
  # first_col<-as.matrix(L[,1])
  # second_col<-as.matrix(L[,2])
  first_fixed_pair<-L[,1:2]
  res_cols1<-c(3:ncol(L))
  comb1<-t(combn(res_cols1,2))
  first_diff<-diff(first_fixed_pair)
  ############
  for(i in 1:nrow(comb1)){
    ref_mat1<-L[,c(comb1[i,])]
    if(same_pairs(first_fixed_pair,ref_mat1)==TRUE){
      if(identical(first_diff,diff(ref_mat1))==TRUE){
        break
      }
    }
  }

  ##second pair of column fix for P2
  # first_col<-as.matrix(L[,1])
  # third_col<-as.matrix(L[,3])
  second_fixed_pair<-L[,c(1,3)]
  res_cols2<-c(2,4:ncol(L))
  second_diff<-diff(second_fixed_pair)
  comb2<-t(combn(res_cols2,2))
  ############
  for(i in 1:nrow(comb2)){
    ref_mat2<-L[,c(comb2[i,])]
    if(same_pairs(second_fixed_pair,ref_mat2)==TRUE){
      if(identical(second_diff,diff(ref_mat2))==TRUE){
        break
      }
    }
  }
  #########P1 and P2 mats
  P1<-rbind(cbind(first_fixed_pair[,1],ref_mat1[,2]*-1),
            cbind(first_fixed_pair[,2]*-1,ref_mat1[,1]))
  P2<-rbind(cbind(second_fixed_pair[,1],ref_mat2[,2]*-1),
            cbind(second_fixed_pair[,2]*-1,ref_mat2[,1]))
  ############################
  ###Superimposing
  design
  inc
  ##for first case
  a<-first_fixed_pair[,1]
  b<-first_fixed_pair[,2]*-1
  d<-ref_mat1[,1]
  c<-ref_mat1[,2]*-1
  mat1<-cbind(a,b,c,d)
  ###superimposing in first case
  w1_list<-list()
  for(j in 1:ncol(mat1)){
    inc2<-inc
    for(i in 1:nrow(design)){
      inc2[c(design[i,]),i]<-mat1[,j]
    }
    w1_list<-append(w1_list,list(inc2))
  }
  ##for second case
  A<-second_fixed_pair[,1]
  B<-second_fixed_pair[,2]*-1
  D<-ref_mat2[,1]
  C<-ref_mat2[,2]*-1
  mat2<-cbind(A,B,C,D)
  ############
  ###superimposing in second case
  w2_list<-list()
  for(j in 1:ncol(mat2)){
    inc2<-inc
    for(i in 1:nrow(design)){
      inc2[c(design[i,]),i]<-mat2[,j]
    }
    w2_list<-append(w2_list,list(inc2))
  }
  #############Now W's final form
  W1<-cbind(rbind(w1_list[[1]],w1_list[[2]]),
            rbind(w1_list[[3]],w1_list[[4]]))
  W2<-cbind(rbind(w2_list[[1]],w2_list[[2]]),
            rbind(w2_list[[3]],w2_list[[4]]))

  ###############
  W_mats<-list(W1,W2)
  #apply(W1,2,sum)
  #interproductsum<-list()
  #indices<-list()
  w11_1<-w1_list[[1]]
  w12_1<-w1_list[[3]]
  w21_1<-w1_list[[2]]
  w22_1<-w1_list[[4]]
  ############
  w11_2<-w2_list[[1]]
  w12_2<-w2_list[[3]]
  w21_2<-w2_list[[2]]
  w22_2<-w2_list[[4]]
  ###################
  interproductsum<-w11_1*w11_2+w12_1*-w12_2+w21_1*w21_2+w22_1*-w22_2
  interproductsum<-apply(interproductsum,1,sum)
  new_list1 <- lapply(W_mats, function(W) {
    list(
      w = W,
      row_sum = rowSums(W),
      col_sum = colSums(W)
    )
  })

  #new_list<-split(interproductsum,rep(1:(length(interproductsum)/2),each=2))
  lm<-list("W matrices",new_list1,"Interproduct Sums",interproductsum)
  return(lm)
}

#' M Array Method for Optimal Covariate Designs (OCDs)
#'
#' @param design Input a design in matrix format and block size k such that (k+1) is a prime number.
#'@description Consider Mutually Orthogonal Latin squares of order s x s, extract first column from it and make a new matrix called Initial block sequence matrix L of order s x s-1. Remove the last row from the L matrix and obtain the incidence matrix of it keeping zeros to the positions corresponding to the elements that were present in the deleted row of L, and then remove the row that contains only non-zero elements, the square matrix thus formed is the M matrix. From the columns of M matrix, choose((s-1),2) pairs are possible. Each of these column pairs is then superimposed to N. Through this method choose((s-1),2) W matrices can be developed.
#'
#' @returns Generates W matrices and Inter product sums of W matrices.
#'@author
#'Neethu RS
#'
#'Cini Varghese
#'
#'Mohd Harun
#'
#'Anindita Datta
#'
#'Ashutosh Dalal
#' @references
#' Das, K., N. K. Mandal, and B. K. Sinha. (2003) <https://doi.org/10.1016/S0378-3758(02)00151-9>. Optimal experimental designs for models with covariates. Journal of Statistical Planning and Inference 115(1): 273-285.
#'
#' Bansal, N., and D. K. Garg. (2022)<https://doi.org/10.1007/s42519-022-00244-0>. Optimum covariate designs for three associate PBIB designs. Journal of Statistical Theory and Practice 16(3): 1-15.
#'
#' @examples
#' library(SFOCDs)
#' mat<-matrix(c(1,2,3,4,1,2,4,5,1,3,4,5,2,3,4,5),nrow=4,byrow=TRUE)
#' MOCDs(mat)
#' @export
MOCDs<-function(design){
  # the input complentary design
  design<-as.matrix(design)
  #####incidence matrix
  k=ncol(design)
  block <- nrow(design)
  trt <- max(design)
  #######incidence matrix
  incidence_matrix <- matrix(0, nrow = block, ncol = trt)
  for (i in 1:block) {
    for (j in 1:ncol(design)) {
      incidence_matrix[i, design[i, j]] <- 1
    }
  }
  incidence_matrix_transpose <- t(incidence_matrix)
  inc<-incidence_matrix_transpose

  ####initial block
  MOLS<-function(v){
    MOLS<-list()
    for(i in 1:(v-1)){
      mols<-NULL
      seq<-c(1:v)
      j=0
      repeat{
        mols<-rbind(mols,c(seq+j))
        j=j+i
        if(nrow(mols)==v){
          mols=mols%%v
          mols[mols==0]<-v
          MOLS<-append(MOLS,list(mols))
          break
        }
      }
    }
    return(MOLS)
  }
  mols<-MOLS(k+1)
  del_col_mols<-lapply(mols,function(mat)mat[,1])
  L<-do.call(cbind,del_col_mols)

  L_del <- L[-(nrow(L)-1), ]

  # Number of treatments (v) and blocks (b)

  b <- ncol(L)
  v <- max(L)
  # Initialize  matrix (v x b)####### m matrix
  m1 <- matrix(0, nrow = v, ncol = b)

  # Fill incidence matrix
  for (j in 1:b) {
    for (i in 1:nrow(L_del)) {
      treatment <- L_del[i, j]
      m1[treatment, j] <- 1
    }
  }
  m2 <- m1[apply(m1, 1, function(x) any(x == 0)), ]

  #print("Matrix after deleting rows with no zeros:")
  #print(m2)
  m <- ifelse(m2 == 0, -1, m2)
  col_pairs <- combn(ncol(m), 2, simplify = FALSE)
  mpair <- do.call(cbind, lapply(col_pairs, function(pair) {
    m[, pair[1]] * m[, pair[2]]
  }))

  W_mats<-list()
  pos_ist<-list()
  for(i in 1:ncol(mpair)){
    minus_pos<-which(mpair[,i]==-1)
    pos_ist<-append(pos_ist,list(design[,c(minus_pos)]))
  }
  incid<-incidence_matrix_transpose

  for(i in 1:length(pos_ist)){
    for(j in 1:nrow(pos_ist[[i]])){
      incid[c(pos_ist[[i]][j,]),j]<--1
    }
    W_mats<-append(W_mats,list(incid))
    incid<-incidence_matrix_transpose
  }

  for (i in seq_along(m)) {

    # skip if m[[i]] is NULL, has no rows, or not a data.frame/matrix
    if (is.null(m[[i]]) || !is.matrix(m[[i]]) && !is.data.frame(m[[i]]) || nrow(as.data.frame(m[[i]])) == 0) {
      next
    }

    for (j in 1:nrow(as.data.frame(m[[i]]))) {
      all_pos_incidence <- abs(m[[i]][j, ])
      incd[c(all_pos_incidence), j] <- incd[c(all_pos_incidence), j] * -1
    }

    W_mats <- append(W_mats, list(incd))
  }
  interproductsum<-list()
  #indices<-list()
  for(i in 1:(length(W_mats)-1)){
    for(j in (i+1):length(W_mats)){
      AxB<-W_mats[[i]]*W_mats[[j]]
      intprdsum<-AxB%*%rep(1,ncol(AxB))
      interproductsum<-append(interproductsum,list(intprdsum,c(i,j)))
      #indices<-append(indices,)
    }
  }
  new_list1 <- lapply(W_mats, function(W) {
    list(
      w = W,
      row_sum = rowSums(W),
      col_sum = colSums(W)
    )
  })

  new_list<-split(interproductsum,rep(1:(length(interproductsum)/2),each=2))
  lm<-list("W matrices",new_list1,"Interproduct Sums",new_list)
  return(lm)
}

#' Orthogonal Integer Array Method for Optimal Covariate Designs (OCDs)
#'
#' @param design Input a design in matrix format and block size k such that it is an odd number.
#'@description Consider OIA of order same as the block size of the required design. Superimpose each array separately into the incidence matrix (N) to get the W matrices. The maximum number of W matrices depends on the column order of OIA.
#'
#' @returns Generates W matrices and Inter product sums of W matrices.
#'@author
#'Neethu RS
#'
#'Cini Varghese
#'
#'Mohd Harun
#'
#'Anindita Datta
#'
#'Ashutosh Dalal
#' @references
#' Das, K., N. K. Mandal, and B. K. Sinha. (2003) <https://doi.org/10.1016/S0378-3758(02)00151-9>. Optimal experimental designs for models with covariates. Journal of Statistical Planning and Inference 115(1): 273-285.
#'
#' Bansal, N., and D. K. Garg. (2022)<https://doi.org/10.1007/s42519-022-00244-0>. Optimum covariate designs for three associate PBIB designs. Journal of Statistical Theory and Practice 16(3): 1-15.
#'
#' @examples
#' library(SFOCDs)
#' mat<-matrix(c(1,2,3,1,2,4,1,2,5,1,3,4,1,3,5,1,4,5,2,3,4,2,3,5,2,4,5,3,4,5),nrow=10,byrow=TRUE)
#' OIAOCDs(mat)
#' @export
OIAOCDs<-function(design){
design<-as.matrix(design)
block <- nrow(design)
trt <- max(design)
incidence_matrix <- matrix(0, nrow = block, ncol = trt)
for (i in 1:block) {
  for (j in 1:ncol(design)) {
    incidence_matrix[i, design[i, j]] <- 1
  }
}
incidence_matrix_transpose <- t(incidence_matrix)
inc<-incidence_matrix_transpose

#####oia
orthogonal_integer_array <- function(n) {
  # Construct orthogonal basis with row sum zero
  # Use Helmert matrix
  H <- matrix(0, n, n-1)
  for (k in 1:(n-1)) {
    H[1:k, k]   <- 1
    H[k+1, k]   <- -k
  }
  return(H)
}
oia<-orthogonal_integer_array(ncol(design))

####superimpose
num_w <- ncol(oia)

# List to store W matrices
W_mats <- vector("list", ncol(oia))

for (w_idx in 1:ncol(oia)) {
  W <- inc  # start with incidence matrix

  # Loop over blocks and positions
  for (block in 1:nrow(design)) {
    treatments <- design[block, ]
    for (pos in seq_along(treatments)) {
      treatment <- treatments[pos]
      W[treatment, block] <- oia[pos, w_idx]
    }
  }

  W_mats[[w_idx]] <- W
}
interproductsum<-list()
#indices<-list()
for(i in 1:(length(W_mats)-1)){
  for(j in (i+1):length(W_mats)){
    AxB<-W_mats[[i]]*W_mats[[j]]
    intprdsum<-AxB%*%rep(1,ncol(AxB))
    interproductsum<-append(interproductsum,list(intprdsum,c(i,j)))
    #indices<-append(indices,)
  }
}
new_list1 <- lapply(W_mats, function(W) {
  list(
    w = W,
    row_sum = rowSums(W),
    col_sum = colSums(W)
  )
})

new_list<-split(interproductsum,rep(1:(length(interproductsum)/2),each=2))
lm<-list("W matrices",new_list1,"Interproduct Sums",new_list)
return(lm)
}

#' Treatment Position Vs Treatment Scatter Plot
#'
#' @param design Input a design in matrix format
#'@description
#'The function will give the scatter plot showing the spread offered by design points in the experimental space. The x axis of the plot represent treatments and y axis the position of treatments in each block. Different colors in the dot represent the blocks.
#'@importFrom graphics axis
#'@importFrom utils combn
#' @returns Generates scatter plot of treatment position Vs treatment
#' @export
#'
#' @examples
#' library(SFOCDs)
#' mat<-matrix(c(  1,  4,  2,  5,
#' 2,  5,  3,  6,
#' 3,  6,  1,  4,
#' 4,  1,  5,  2,
#' 5,  2,  6,  3,
#' 6,  3,  4,  1),nrow=6,byrow=TRUE)
#' MaxDot(mat)
MaxDot<-function(design){
  mat<-as.matrix(design)
  final_mat<-NULL
  for(i in 1:nrow(mat)){
    for(j in 1:ncol(mat)){
      final_mat=rbind(final_mat,c(mat[i,j],j,i))
    }
  }
  final_mat
  colnames(final_mat)<-c("Treatment","Position","Block")
  Array<-final_mat

  #plot

  aray<-Array[order(Array[,3]),]
  # Define colors for each category
  b=nrow(mat)
  color_the_blocks<-function(matrix,b){
    matrix<-as.matrix(matrix)
    k<-nrow(matrix)/b
    colors <- c("red", "blue", "green", "purple", "orange","violet","wheat","yellow",
                "turquoise","skyblue","salmon","black","pink","peru","palegreen","navy","hotpink","gold",
                "firebrick","azure")
    seq<-colors[1:b]
    new_seq<-rep(seq,each=k)
    pch_seq<-rep(1:b,each=k)
    # Create a scatter plot with different colors for different categories
    plot(matrix[,2], matrix[,1], xlab="Position", ylab="Treatment", col = new_seq,xaxt="n", pch = 16,cex=2,bg=, main = "Scatter Plot ")
    axis(1,at=seq(1,b,by=1),labels=seq(1,b,by=1))
  }
  color_the_blocks(aray,b)

}

#' Maxpro Criterion for Treatment vs Position
#'
#' @param design Input a design in matrix format
#'@description
#'User input should be the original design and this function
#'automatically convert the design in treatment vs position 2D array and then
#'print the Maxpro Criterion value.
#'
#' @returns Maxpro criterion value for a given design
#' @export
#'
#' @examples
#' library(SFOCDs)
#' mat<-matrix(c(
#' 1,2,3,
#' 2,1,4,
#' 3,4,1,
#' 4,3,2),nrow=4,byrow=TRUE)
#' Maxpro_Criterion(mat)

Maxpro_Criterion<-function(design){
  d<-as.matrix(design)
  #Convert design to proper format
  convert_des<-function(design){
    mat<-as.matrix(design)
    final_mat<-NULL
    for(i in 1:nrow(mat)){
      for(j in 1:ncol(mat)){
        final_mat=rbind(final_mat,c(mat[i,j],j,i))
      }
    }
    return(final_mat[,-ncol(final_mat)])
  }
  ##########

  #####Maxpro_criterion

  MP<-function(design){
    #design<-read.table("clipboard",sep='\t')
    d<-as.matrix(design)
    v=max(d)
    k=ncol(d)
    sumsq<-NULL
    for(i in 1:(nrow(d)-1)){
      j=i+1
      while(j<=nrow(d)){
        sumsq=c(sumsq,(1/prod((abs(d[i,1]-d[j,1])+(1/v))^2,((abs(d[i,2]-d[j,2])+(1/k)))^2)))
        j=j+1
      }
    }
    value<-(sum(sumsq)/choose(nrow(d),2))^(1/2)
    value
    return(value)
  }
  ######input the design for which maxpro want to find
  des<-convert_des(d)

  #########Find Maxpro value of converted design
  return(MP(des))
}

#' Hadamard Method for Optimal Covariate Designs (OCDs)
#'
#' @param design Input a design in matrix format and block size k is multiple of 4.
#'@description Construct Hadamard matrix H_k = (1, h_1 , h_2,..., h_k-1) where k is the block size of the required design. Then superimpose each columns of H_k leaving the first column which is in natural order separately into the N matrix to get the W matrices. The maximum number of W matrices will be k-1.
#'
#' @returns Generates W matrices and Inter product sums of W matrices.
#' @export
#'@author
#'Neethu RS
#'
#'Cini Varghese
#'
#'Mohd Harun
#'
#'Anindita Datta
#'
#'Ashutosh Dalal
#' @references
#' Das, K., N. K. Mandal, and B. K. Sinha. (2003) <https://doi.org/10.1016/S0378-3758(02)00151-9>. Optimal experimental designs for models with covariates. Journal of Statistical Planning and Inference 115(1): 273-285.
#'
#' Bansal, N., and D. K. Garg. (2022)<https://doi.org/10.1007/s42519-022-00244-0>. Optimum covariate designs for three associate PBIB designs. Journal of Statistical Theory and Practice 16(3): 1-15.
#'
#' @examples
#' library(SFOCDs)
#' mat<-matrix(c(1,2,3,4,1,2,4,5,1,3,4,5,2,3,4,5),nrow=4,byrow=TRUE)
#' HadamardOCDs(mat)
HadamardOCDs<-function(design){
  ####incidence matrix
  design<-as.matrix(design)
  block <- nrow(design)
  trt <- max(design)
  #######incidence matrix
  incidence_matrix <- matrix(0, nrow = block, ncol = trt)
  for (i in 1:block) {
    for (j in 1:ncol(design)) {
      incidence_matrix[i, design[i, j]] <- 1
    }
  }
  incidence_matrix_transpose <- t(incidence_matrix)
  ######

  ##Hadamard##
  # Function to generate Hadamard matrix of order k
  hadamard_matrix <- function(k) {
    if (k == 1) {
      return(matrix(1, nrow = 1, ncol = 1))  # Base case, H_1 = [1]
    } else {
      # Recursively build the Hadamard matrix for order 2^n
      H <- hadamard_matrix(k / 2)

      # Combine matrices to form the Hadamard matrix of order k
      top_left <- H
      top_right <- H
      bottom_left <- H
      bottom_right <- -H

      # Combine the blocks into a larger matrix
      return(rbind(
        cbind(top_left, top_right),
        cbind(bottom_left, bottom_right)
      ))
    }
  }
  k <- ncol(design)
  H1 <- hadamard_matrix(k)
  H<-H1[,-1]


  #####super imposing part
  W_mats<-list()
  pos_ist<-list()
  for(i in 1:ncol(H)){
    minus_pos<-which(H[,i]==-1)
    pos_ist<-append(pos_ist,list(design[,c(minus_pos)]))
  }
  #######
  incid<-incidence_matrix_transpose

  for(i in 1:length(pos_ist)){
    for(j in 1:nrow(pos_ist[[i]])){
      incid[c(pos_ist[[i]][j,]),j]<--1
    }
    W_mats<-append(W_mats,list(incid))
    incid<-incidence_matrix_transpose
  }
  ######################
  in_H_negative_pos<-NULL
  for(i in 1:ncol(H)){
    in_H_negative_pos<-rbind(in_H_negative_pos, which(H[,i]==-1))
  }
  ####################
  position_mat_list<-list()
  for(j in 1:nrow(in_H_negative_pos)){
    pos_mat<-design[,c(in_H_negative_pos[j,])]
    position_mat_list<-append(position_mat_list,list(pos_mat))
  }
  ##################3
  all_w_mats<-list()
  for(i in 1:length(position_mat_list)){
    incid1<-incid
    for(j in 1:nrow(position_mat_list[[i]])){
      #for(k in 1:nrow(position_mat_list[[i]])){
      incid1[c(position_mat_list[[i]][j,]),j]<--1
    }
    all_w_mats<-append(all_w_mats,list(incid1))
  }
  ##################################
  W_mats<-all_w_mats
  interproductsum<-list()
  for(i in 1:(length(W_mats)-1)){
    for(j in (i+1):length(W_mats)){
      AxB<-W_mats[[i]]*W_mats[[j]]
      intprdsum<-AxB%*%rep(1,ncol(AxB))
      interproductsum<-append(interproductsum,list(intprdsum,c(i,j)))
      #indices<-append(indices,)
    }
  }
  new_list1 <- lapply(W_mats, function(W) {
    list(
      w = W,
      row_sum = rowSums(W),
      col_sum = colSums(W)
    )
  })

  new_list<-split(interproductsum,rep(1:(length(interproductsum)/2),each=2))
  lm<-list("W matrices",new_list1,"Interproduct Sums",new_list)
  return(lm)
}

#hadmardOCDs(mat)

#' Reshuffle Design
#'
#' @param design Input any base design
#'@param type Input 1 or 2. By default set 1. If type =2 then reshuffled for JuxtaOCDs.
#' @returns Reshuffled the base design such that each column has minimum possible repeatations of symbols.
#' @export
#'
#' @examples
#' library(SFOCDs)
#' mat<-matrix(c(1,2,3,4,1,2,4,5,1,3,4,5,2,3,4,5),nrow=4,byrow=TRUE)
#' reshuffle_des(mat,type=1)

reshuffle_des<-function(design,type=1){
  design=as.matrix(design)
  if(type==1){
    reshuffled_mat <- function(design) {

      b <- nrow(design)
      k <- ncol(design)

      # replication of each treatment
      trt <- as.vector(design)
      r <- table(trt)[1]   # assuming equal replication

      beta <- r / k

      result <- matrix(NA, b, k)

      fill_row <- function(i) {
        if (i > b) return(TRUE)

        perms <- gtools::permutations(k, k, design[i, ])

        for (p in 1:nrow(perms)) {

          candidate <- perms[p, ]
          valid <- TRUE

          for (j in 1:k) {

            count <- sum(result[, j] == candidate[j], na.rm = TRUE)

            if (count >= beta) {
              valid <- FALSE
              break
            }
          }

          if (valid) {
            result[i, ] <<- candidate

            if (fill_row(i + 1)) return(TRUE)

            result[i, ] <<- NA
          }
        }

        return(FALSE)
      }

      success <- fill_row(1)

      if (!success) stop("No valid arrangement exists")

      return(result)
    }
  }else{
reshuffled_mat <- function(design) {
  k <- ncol(design)
  target <- k + 1

  new_design <- matrix(NA, nrow=nrow(design), ncol=k)
  pos_list <- list()

  # First row fixed
  new_design[1, ] <- design[1, ]

  for (j in 1:k) {
    pos_list[[as.character(design[1, j])]] <- j
  }

  # Process remaining rows
  for (i in 2:nrow(design)) {
    row <- design[i, ]
    new_row <- rep(NA, k)
    used_pos <- rep(FALSE, k)

    # Step 1: try complementary placement
    for (j in 1:k) {
      trt <- row[j]
      trt_char <- as.character(trt)

      if (!is.null(pos_list[[trt_char]])) {
        pos <- target - pos_list[[trt_char]]

        if (!used_pos[pos]) {
          new_row[pos] <- trt
          used_pos[pos] <- TRUE
        }
      }
    }

    # Step 2: fill remaining safely
    for (j in 1:k) {
      trt <- row[j]
      if (!(trt %in% new_row)) {
        free_pos <- which(!used_pos)[1]
        new_row[free_pos] <- trt
        used_pos[free_pos] <- TRUE

        # store position if first time
        trt_char <- as.character(trt)
        if (is.null(pos_list[[trt_char]])) {
          pos_list[[trt_char]] <- free_pos
        }
      }
    }

    new_design[i, ] <- new_row
  }

  return(new_design)
}
  }
  return(reshuffled_mat(design))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reshuffle_des.R
\name{reshuffle_des}
\alias{reshuffle_des}
\title{Reshuffle Design}
\usage{
reshuffle_des(design, type = 1)
}
\arguments{
\item{design}{Input any base design}

\item{type}{Input 1 or 2. By default set 1. If type =2 then reshuffled for JuxtaOCDs.}
}
\value{
Reshuffled the base design such that each column has minimum possible repeatations of symbols.
}
\description{
Reshuffle Design
}
\examples{
library(SFOCDs)
mat<-matrix(c(1,2,3,4,1,2,4,5,1,3,4,5,2,3,4,5),nrow=4,byrow=TRUE)
reshuffle_des(mat,type=1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/OIAOCDs.R
\name{OIAOCDs}
\alias{OIAOCDs}
\title{Orthogonal Integer Array Method for Optimal Covariate Designs (OCDs)}
\usage{
OIAOCDs(design)
}
\arguments{
\item{design}{Input a design in matrix format and block size k such that it is an odd number.}
}
\value{
Generates W matrices and Inter product sums of W matrices.
}
\description{
Consider OIA of order same as the block size of the required design. Superimpose each array separately into the incidence matrix (N) to get the W matrices. The maximum number of W matrices depends on the column order of OIA.
}
\examples{
library(SFOCDs)
mat<-matrix(c(1,2,3,1,2,4,1,2,5,1,3,4,1,3,5,1,4,5,2,3,4,2,3,5,2,4,5,3,4,5),nrow=10,byrow=TRUE)
OIAOCDs(mat)
}
\references{
Das, K., N. K. Mandal, and B. K. Sinha. (2003) <https://doi.org/10.1016/S0378-3758(02)00151-9>. Optimal experimental designs for models with covariates. Journal of Statistical Planning and Inference 115(1): 273-285.

Bansal, N., and D. K. Garg. (2022)<https://doi.org/10.1007/s42519-022-00244-0>. Optimum covariate designs for three associate PBIB designs. Journal of Statistical Theory and Practice 16(3): 1-15.
}
\author{
Neethu RS

Cini Varghese

Mohd Harun

Anindita Datta

Ashutosh Dalal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MaxDot.R
\name{MaxDot}
\alias{MaxDot}
\title{Treatment Position Vs Treatment Scatter Plot}
\usage{
MaxDot(design)
}
\arguments{
\item{design}{Input a design in matrix format}
}
\value{
Generates scatter plot of treatment position Vs treatment
}
\description{
The function will give the scatter plot showing the spread offered by design points in the experimental space. The x axis of the plot represent treatments and y axis the position of treatments in each block. Different colors in the dot represent the blocks.
}
\examples{
library(SFOCDs)
mat<-matrix(c(  1,  4,  2,  5,
2,  5,  3,  6,
3,  6,  1,  4,
4,  1,  5,  2,
5,  2,  6,  3,
6,  3,  4,  1),nrow=6,byrow=TRUE)
MaxDot(mat)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MOCDs.R
\name{MOCDs}
\alias{MOCDs}
\title{M Array Method for Optimal Covariate Designs (OCDs)}
\usage{
MOCDs(design)
}
\arguments{
\item{design}{Input a design in matrix format and block size k such that (k+1) is a prime number.}
}
\value{
Generates W matrices and Inter product sums of W matrices.
}
\description{
Consider Mutually Orthogonal Latin squares of order s x s, extract first column from it and make a new matrix called Initial block sequence matrix L of order s x s-1. Remove the last row from the L matrix and obtain the incidence matrix of it keeping zeros to the positions corresponding to the elements that were present in the deleted row of L, and then remove the row that contains only non-zero elements, the square matrix thus formed is the M matrix. From the columns of M matrix, choose((s-1),2) pairs are possible. Each of these column pairs is then superimposed to N. Through this method choose((s-1),2) W matrices can be developed.
}
\examples{
library(SFOCDs)
mat<-matrix(c(1,2,3,4,1,2,4,5,1,3,4,5,2,3,4,5),nrow=4,byrow=TRUE)
MOCDs(mat)
}
\references{
Das, K., N. K. Mandal, and B. K. Sinha. (2003) <https://doi.org/10.1016/S0378-3758(02)00151-9>. Optimal experimental designs for models with covariates. Journal of Statistical Planning and Inference 115(1): 273-285.

Bansal, N., and D. K. Garg. (2022)<https://doi.org/10.1007/s42519-022-00244-0>. Optimum covariate designs for three associate PBIB designs. Journal of Statistical Theory and Practice 16(3): 1-15.
}
\author{
Neethu RS

Cini Varghese

Mohd Harun

Anindita Datta

Ashutosh Dalal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Maxpro_Criterion.R
\name{Maxpro_Criterion}
\alias{Maxpro_Criterion}
\title{Maxpro Criterion for Treatment vs Position}
\usage{
Maxpro_Criterion(design)
}
\arguments{
\item{design}{Input a design in matrix format}
}
\value{
Maxpro criterion value for a given design
}
\description{
User input should be the original design and this function
automatically convert the design in treatment vs position 2D array and then
print the Maxpro Criterion value.
}
\examples{
library(SFOCDs)
mat<-matrix(c(
1,2,3,
2,1,4,
3,4,1,
4,3,2),nrow=4,byrow=TRUE)
Maxpro_Criterion(mat)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/JuxtaOCDs.R
\name{JuxtaOCDs}
\alias{JuxtaOCDs}
\title{Juxtaposed Method for Optimal Covariate Designs (OCDs)}
\usage{
JuxtaOCDs(design)
}
\arguments{
\item{design}{Input a design in matrix format and block size k such that (k+1) is a prime number.}
}
\value{
Generates W matrices and Inter product sums of W matrices.
}
\description{
Consider L matrix, construct resolvable sets by grouping columns into pairs that have the same ordered set of elements. For each pair, arrange the two column-sets horizontally (2(s-1)) and change the signs of any two sets. This new setup form the P_i matrix of order 2(s-1) x 2. Then superimpose the first column of P_i onto N and that produce W_i, where first set of order v x b will be W_i^11 and other set below is W_i^21. Likewise use second column of P_i  to  get W_i^12 and W_i^22.  Repeat for every P_i to get collection of W_i's. The grand total of Hadamard product of all W_i^ij will be zero provided a foldover of any one of the W_i^ij is taken.
}
\examples{
library(SFOCDs)
mat1<-matrix(c(
1,2,3,4,5,6,
7,8,9,10,11,1,
12,13,14,15,2,7,
16,17,18,3,8,12,
19,20,4,9,13,16,
21,5,10,14,17,19,
6,11,15,18,20,21),nrow=7,byrow=TRUE)
JuxtaOCDs(mat1)
}
\references{
Das, K., N. K. Mandal, and B. K. Sinha. (2003) <https://doi.org/10.1016/S0378-3758(02)00151-9>. Optimal experimental designs for models with covariates. Journal of Statistical Planning and Inference 115(1): 273-285.

Bansal, N., and D. K. Garg. (2022)<https://doi.org/10.1007/s42519-022-00244-0>. Optimum covariate designs for three associate PBIB designs. Journal of Statistical Theory and Practice 16(3): 1-15.
}
\author{
Neethu RS

Cini Varghese

Mohd Harun

Anindita Datta

Ashutosh Dalal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/HadamardOCDs.R
\name{HadamardOCDs}
\alias{HadamardOCDs}
\title{Hadamard Method for Optimal Covariate Designs (OCDs)}
\usage{
HadamardOCDs(design)
}
\arguments{
\item{design}{Input a design in matrix format and block size k is multiple of 4.}
}
\value{
Generates W matrices and Inter product sums of W matrices.
}
\description{
Construct Hadamard matrix H_k = (1, h_1 , h_2,..., h_k-1) where k is the block size of the required design. Then superimpose each columns of H_k leaving the first column which is in natural order separately into the N matrix to get the W matrices. The maximum number of W matrices will be k-1.
}
\examples{
library(SFOCDs)
mat<-matrix(c(1,2,3,4,1,2,4,5,1,3,4,5,2,3,4,5),nrow=4,byrow=TRUE)
HadamardOCDs(mat)
}
\references{
Das, K., N. K. Mandal, and B. K. Sinha. (2003) <https://doi.org/10.1016/S0378-3758(02)00151-9>. Optimal experimental designs for models with covariates. Journal of Statistical Planning and Inference 115(1): 273-285.

Bansal, N., and D. K. Garg. (2022)<https://doi.org/10.1007/s42519-022-00244-0>. Optimum covariate designs for three associate PBIB designs. Journal of Statistical Theory and Practice 16(3): 1-15.
}
\author{
Neethu RS

Cini Varghese

Mohd Harun

Anindita Datta

Ashutosh Dalal
}
